/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef EAM_C_CONTEXT_H_
#define EAM_C_CONTEXT_H_


/* Set our stack bottom pointer approximation to a user-supplied value.
   This is useful when the user can't initialize in our funny
   'continuation' style by supplying a pointer to the function used to
   execute the rest of the program. */
void eam_save_stack_bottom(void *stack_bottom);

/* Save the current stack bottom and execute the given function, which
   may assume (only in its body) that the stack bottom was appropriately
   saved: */
void eam_save_stack_bottom_and_continue(void (*function_from_the_user)(void));

/* Execute the given function, which is given as parameters the stack
   bottom (which must have been saved before) and the stack top: */
void eam_call_with_stack_bottom_and_stack_top(void (*function)(const void *stack_bottom,
                                                           const void *stack_top));

/* A version of eam_save_stack_bottom_and_continue() with a slightly different
   interface which allows the user to just set the stack bottom in main()
   and then jump to a 'fake' procedure simulating main(). The valure
   retuned from the user-supplied function is returned.
   This machinery is easy to hide behind preprocessor macros, so that it's
   possible for the stack bottom to be always implicitly saved at startup
   time. */
int eam_save_stack_bottom_and_call_main(
       int (*main_function_from_the_user)(int argc, char **argv),
       int argc,
       char **argv);

/* Call the given function which is given as parameters two pointers
   referring the first and the last word of a machine registers image: */
void eam_call_with_first_register_and_last_register(void (*function)(
        const void *first_register,
        const void *last_register));

#endif // #ifndef EAM_C_CONTEXT_H_
