/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "environment.h"
#include <stdlib.h>

/* Return getenv(variable_name) if it's not NULL, and an empty string
   otherwise. The returned string should never be deallocated by the
   caller: */
const char* eam_getenv_with_empty_default(const char *variable_name){
  static const char *empty_string = "";
  return eam_getenv_with_default(variable_name, empty_string);
}

/* Return getenv(variable_name) if it's not NULL, and the given default
   strign otherwise. The returned string should never be deallocated by the
   caller. default_string must remain alive until the last use of the
   result of this function. */
const char* eam_getenv_with_default(const char *variable_name,
                                    const char *default_string){
  char *variable_value = getenv(variable_name);
  if(variable_value == NULL)
    return default_string;
  else
    return variable_value;
}
