/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2005 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "types.h"

/* A really trivial implementation of lists: */

/* Let's make list an "abstract type", or the most similar thing we can have
   in C: */
struct eam_list;

/* The empty list. This is guaranteed to be equal to NULL. */
extern struct eam_list *eam_nil;

/* Return true iff the list is empty: */
int eam_null(const struct eam_list *list);

/* Return true iff the list is not empty: */
int eam_neam_null(const struct eam_list *list);

/* Prepend object to list making a new list which is returned. list is not
   updated. The new element is malloc()ed. */
struct eam_list* eam_cons(void *object, struct eam_list* list) __attribute__(( malloc ));

/* Selectors: */
void* eam_car(const struct eam_list* list);
struct eam_list* eam_cdr(const struct eam_list* list);

/* Updaters: */
void set_eam_car(struct eam_list* list, void *object);
void set_eam_cdr(struct eam_list* list, struct eam_list *object);

/* Return the length of a list: */
int eam_length(const struct eam_list* list);

/* Return a new list whose elements are the the same as the ones in the given
   list, but in the reverse order. The given list is not altered, and pointed
   elements are shared by the returned structure: */
struct eam_list* eam_reverse(const struct eam_list *list);

/* Destroy all the conses, but not the pointed elements. This does not work
   with circular lists or with lists having non-free()able elements: */
void eam_destroy_list_structure(const struct eam_list *list);

/* Destroy all the conses *and* the pointed elements, which must have
   been malloc()ed. This does not work with circular lists or with lists
   having non-free()able elements: */
void eam_destroy_whole_list(const struct eam_list *list);
