/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <assert.h>
#include <malloc.h>
#include <string.h>

#include "../../common/types.h"

word_t do_a_lot_of_stuff_in_another_compilation_unit(const word_t x, 
                                                     const integer_t increment,
                                                     ...)
  __attribute__(( noinline ));
/* GCC isn't currently able to inline this, but we want to be safe now and
   in the future. */

word_t do_a_lot_of_stuff_in_another_compilation_unit(const word_t x,
                                                     const integer_t increment,
                                                     ...){
  va_list ap;
  va_start(ap, increment);
  volatile word_t copy_of_x = x;
  integer_t sum_of_variadic_parameters = 0;

  printf("x is "INTEGER_T_FORMAT" (as a signed integer)\n", (integer_t)x);
  printf("x is %p (as a pointer)\n", x);
  
  integer_t variadic_parameter;
  do{
    variadic_parameter = va_arg(ap, int);
    sum_of_variadic_parameters += variadic_parameter;
  }while(variadic_parameter != 0);
  va_end(ap);

  /* Array stress-test: */  
#define BUFFER_ELEMENTS_NO 10240
  word_t* buffer = (word_t*)
    malloc(sizeof(word_t) * BUFFER_ELEMENTS_NO);
  word_t* buffer2 = (word_t*)
    malloc(sizeof(word_t) * BUFFER_ELEMENTS_NO);
  word_t* buffer3 = (word_t*)
    malloc(sizeof(word_t) * BUFFER_ELEMENTS_NO);
  assert(buffer != NULL);
  integer_t i;
  for(i = 0; i < BUFFER_ELEMENTS_NO; i++)
    buffer[i] = x;
  memset(buffer2, 0, BUFFER_ELEMENTS_NO * sizeof(word_t));
  memcpy(buffer2, buffer, BUFFER_ELEMENTS_NO * sizeof(word_t));
  memcpy(buffer3, buffer2, BUFFER_ELEMENTS_NO * sizeof(word_t));
  for(i = 0; i < BUFFER_ELEMENTS_NO; i++){
    assert(buffer[i] == x);
    assert(buffer[i] == copy_of_x);
    assert(buffer2[i] == x);
    assert(buffer3[i] == x);
  }
  free(buffer);
  free(buffer2);
  free(buffer3);
    
  assert(x == copy_of_x);
  
  printf("the sum of all variadic parameters is "INTEGER_T_FORMAT"\n",
         sum_of_variadic_parameters);
  
  if(increment < 0)
    do_a_lot_of_stuff_in_another_compilation_unit(x, increment + 1, 1, 2, 3, 0);
  else if(increment > 0)
    do_a_lot_of_stuff_in_another_compilation_unit(x, increment - 1, 1, 2, 3, 0);
  
  return (word_t)(((integer_t)x) + increment);
}

