/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef EAMGC_H_
#define EAMGC_H_

#include "../../common/types.h"
#include "roots.h"
#include "trace.h"
#include "homogeneous.h"
#include "large.h"
#include "generic_pools.h"


/* Initialize the garbage collector, and execute the given function. The
   collector will not work after the given gunction returns. */
void eamgc_initialize_garbage_collector_and_continue(void (*continuation)(void));

/* Initialize the garbage collector and return. Note that this only works
   if the stack bottom is saved (see c_context.h), either before or
   after calling this function: */
void eamgc_initialize_garbage_collector(void);

/* Destroy all the collector data structures. The collector won't be usable
   after calling this function, until explicitly re-initialized. This finalizes
   all alive large objects and frees resources. */
void eamgc_finalize_garbage_collector(void);

/* Perform a collection immediately. Not normally called by the user. */
void eamgc_garbage_collect(void);

/* Disable or re-enable the garbage collector. When the collector is 
   disabled allocating from a full pool always creates a new page. */
void eamgc_enable_garbage_collector(void);
void eamgc_disable_garbage_collector(void);

/* Check whether the GC is currently enabled: */
integer_t eamgc_is_garbage_collector_enabled(void) __attribute__ (( pure ));

/* Return the tag associated to x, which must be a heap object (either
   homogeneous or large): */
unsigned_integer_t eamgc_object_to_tag(const word_t x)
  __attribute__ (( pure ));

/* Return the data associated to x, which must be a heap object (either
   homogeneous or large): */
word_t eamgc_object_to_data(const word_t x)
  __attribute__ (( pure ));

#endif // #ifndef EAMGC_H_
