/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef EAMGC_HEURISTICS_H_
#define EAMGC_HEURISTICS_H_

/* Heuristics allow to decide whether to destroy an empty page or to keep it,
   and whether to allocate new space or to trigger a collection. The
   information they use is based on statistics as they were computed the last
   time (heuristics *don't* automatically compute new statistics); hence
   heuristics are "cheap", and suitable to be tested relatively often. */

#include "../../common/types.h"

/* Should we destroy an empty homogeneous page? Otherwise the page is
   kept, for future use: */
integer_t eamgc_should_we_shrink_the_homogeneous_heap(void)
  __attribute__(( pure ));

/* Should we add a new empty homogeneous page to allocate from? Otherwise
   a GC should be triggered (which, in really pathological cases, could
   free nothing; hence is's possible in theory that we had to create a new
   page also right after a collection, and a very expensive failed
   allocation): */
integer_t eamgc_should_we_enlarge_the_homogeneous_heap(void)
  __attribute__(( pure ));

/* Should we add a new empty large object? Otherwise a GC should be triggered
   (which, in really pathological cases, could free nothing; hence is's
   possible in theory that we had to allocate storage page also right after
   a collection, and a very expensive failed allocation): */
integer_t eamgc_should_we_enlarge_the_large_heap(void)
  __attribute__(( pure ));


#endif // #ifndef EAMGC_HEURISTICS_H_
