/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef EAM_PINS_H_
#define EAM_PINS_H_

#include "../common/types.h"

/* A "pin" on the compilation unit u is an element of the dependency relation
   from some unidentified compilation unit to u. u is initialized when its
   first pin is created, and finalized when the last one is removed. Each 
   compilation unit includes a "pin counter". */

/* Declare a pin counter as an extern global: */
#define EAM_DECLARE_PIN_COUNTER(mangled_name) \
  extern integer_t eam_ ## mangled_name ## _pins_no;

/* Declare and initialize a pin counter, to be allocated as a global: */
#define EAM_IMPLEMENT_PIN_COUNTER(mangled_name) \
  integer_t eam_ ## mangled_name ## _pins_no = (integer_t)0;

/* Add a pin, initializing if needed: */
#define EAM_PIN_COMPILATION_UNIT(mangled_name){ \
  if(++ eam_ ## mangled_name ## _pins_no == (integer_t)1) \
    eam_initialize_ ## mangled_name(); \
}

/* Remove a pin, finalizing if needed: */
#define EAM_UNPIN_COMPILATION_UNIT(mangled_name){ \
  if(-- eam_ ## mangled_name ## _pins_no == (integer_t)0) \
    eam_finalize_ ## mangled_name(); \
}

#endif // #ifndef EAM_PINS_H_
