/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "eamc.h"
#include <string.h>

/* Resolve all the directly named .eaml files (i.e. not the ones
   obtained by resolving dependencies), and return a list of completely
   resolved file names (a list of malloc()ed strings): */
struct eam_list* resolve_eaml_parameters(int argc, char **argv,
                                         const struct eam_list *eaml_search_path){
  /* The list to return: */
  struct eam_list *r = eam_nil;
  
  /* For every argument (we scan the array from the end to the beginning, so
     that we can build a list by prepending)... */
  int i;
  for(i = argc - 1; i > 0; i--){ // we skip argv[0], of course
    /* ...if it has the right extension... */
    char *extension = eam_file_name_to_extension(argv[i]);
    if(! strcmp(extension, "eaml")){
      /* ...then resolve it and add it to the list: */
      char *full_path_name =
        eam_locate_in_search_path(argv[i], eaml_search_path);
      if(full_path_name == NULL)
        eam_command_line_fatal("%s is not in the eAML search path", argv[i]);
      r = eam_cons(full_path_name, r);
    } // if
    else // we currently support only .eaml files as arguments
      eam_command_line_fatal("%s has not the .eaml extension", argv[i]);
    free(extension); // this was malloc()ed
  } // for
  
  /* Return the list we built: */
  return r;
}

/* Resolve all the named library files, and return a list of completely
   resolved file names (a list of malloc()ed strings): */
/* struct eam_list* resolve_library_parameters(int argc, char **argv, */
/*                                             const struct eam_list *repository_search_path){ */
/*   return eam_nil; */
/* } */
