/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2006 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU epsilon; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef EAM_IMPORT_GRAPH_H_
#define EAM_IMPORT_GRAPH_H_

#include "../common/types.h"
#include "../common/list.h"
#include "symbol_table.h"

/* An import graph is a directed acyclic graph whose nodes represent eAM
   compilation units, and whose edges represent the "import" relation. */
struct eam_import_graph;

/* Make a new import graph, with information from the given compilation units
   and the given symbol table. The given data are not copied, and they must be
   retained with no change for the graph lifetime. Fail with a fatal error on
   any cycle or reference to an unknown compilation unit. */
struct eam_import_graph* eam_create_import_graph(const struct eam_list *compilation_units,
					 const struct eam_symbol_table *symbol_table);

/* Destroy an import graph: */
void eam_destroy_import_graph(struct eam_import_graph *import_graph);

/* Add an edge to the graph. Fail with a fatal error if this creates a cycle,
   or if any of the two given nodes is unknown. */
//void add_import(struct eam_import_graph *import_graph,
//		const char *importing_compilation_unit_name,
//		const char *imported_compilation_unit_name);

/* Return a list of newly-allocated struct eam_import_graph_node's, or fail with a
   fatal error if the given compilation unit is not known: */
//struct eam_list* imported_compilation_units(const struct eam_import_graph *import_graph,
//					const char *importing_compilation_unit_name);

#endif // #ifndef EAM_IMPORT_GRAPH_H_
